@@ -1,9 +1,9 @@
 /*
- * $Id: dialog.c,v 1.163 2007/02/25 19:06:11 tom Exp $
+ * $Id: dialog.c,v 1.174 2008/08/19 23:21:36 tom Exp $
  *
  *  cdialog - Display simple dialog boxes from shell scripts
  *
- *  Copyright 2000-2006,2007	Thomas E. Dickey
+ *  Copyright 2000-2007,2008	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -53,6 +53,7 @@ typedef enum {
     ,o_checklist
     ,o_clear
     ,o_colors
+    ,o_column_separator
     ,o_cr_wrap
     ,o_create_rc
     ,o_default_item
@@ -96,8 +97,10 @@ typedef enum {
     ,o_no_shadow
     ,o_nocancel
     ,o_noitem
+    ,o_nook
     ,o_ok_label
     ,o_output_fd
+    ,o_output_separator
     ,o_passwordbox
     ,o_passwordform
     ,o_pause
@@ -105,6 +108,7 @@ typedef enum {
     ,o_print_size
     ,o_print_version
     ,o_progressbox
+    ,o_quoted
     ,o_radiolist
     ,o_screen_center
     ,o_separate_output
@@ -183,6 +187,7 @@ static const Options options[] = {
     { "checklist",	o_checklist,		2, "<text> <height> <width> <list height> <tag1> <item1> <status1>..." },
     { "clear",		o_clear,		1, "" },
     { "colors",		o_colors,		1, "" },
+    { "column-separator",o_column_separator,	1, "<str>" },
     { "cr-wrap",	o_cr_wrap,		1, "" },
     { "create-rc",	o_create_rc,		1, NULL },
     { "default-item",	o_default_item,		1, "<str>" },
@@ -226,11 +231,14 @@ static const Options options[] = {
     { "no-kill",	o_no_kill,		1, "" },
     { "no-label",	o_no_label,		1, "<str>" },
     { "no-lines",	o_no_lines, 		1, "" },
+    { "no-ok",		o_nook,			1, "" },
     { "no-shadow",	o_no_shadow,		1, "" },
     { "nocancel",	o_nocancel,		1, NULL }, /* see --no-cancel */
     { "noitem",		o_noitem,		1, NULL },
+    { "nook",		o_nook,			1, "" }, /* See no-ok */
     { "ok-label",	o_ok_label,		1, "<str>" },
     { "output-fd",	o_output_fd,		1, "<fd>" },
+    { "output-separator",o_output_separator,	1, "<str>" },
     { "passwordbox",	o_passwordbox,		2, "<text> <height> <width> [<init>]" },
     { "passwordform",	o_passwordform,		2, "<text> <height> <width> <form height> <label1> <l_y1> <l_x1> <item1> <i_y1> <i_x1> <flen1> <ilen1>..." },
     { "pause",		o_pause,		2, "<text> <height> <width> <seconds>" },
@@ -238,6 +246,7 @@ static const Options options[] = {
     { "print-size",	o_print_size,		1, "" },
     { "print-version",	o_print_version,	5, "" },
     { "progressbox",	o_progressbox,		2, "<height> <width>" },
+    { "quoted",		o_quoted,		1, "" },
     { "radiolist",	o_radiolist,		2, "<text> <height> <width> <list height> <tag1> <item1> <status1>..." },
     { "screen-center",	o_screen_center,	1, NULL },
     { "separate-output",o_separate_output,	1, "" },
@@ -302,7 +311,7 @@ string_to_argv(char *blob)
 		    inparm = TRUE;
 		}
 	    } else if (blob[n] == '\\') {
-		if (quoted && !isspace(CharOf(blob[n + 1]))) {
+		if (quoted && !isspace(UCH(blob[n + 1]))) {
 		    if (!inparm) {
 			if (pass)
 			    result[count] = param;
@@ -315,7 +324,7 @@ string_to_argv(char *blob)
 		    }
 		}
 		++n;
-	    } else if (!quoted && isspace(CharOf(blob[n]))) {
+	    } else if (!quoted && isspace(UCH(blob[n]))) {
 		inparm = FALSE;
 		if (pass) {
 		    *param++ = '\0';
@@ -420,24 +429,32 @@ unescape_argv(int *argcp, char ***argvp)
 		char **list;
 		char *blob;
 		int added;
+		int bytes_read;
 		int length;
 		int n;
-		struct stat sb;
-
-		if (stat(filename, &sb) == 0
-		    && (sb.st_mode & S_IFMT) == S_IFREG
-		    && sb.st_size != 0) {
-
-		    blob = dlg_malloc(char, sb.st_size + 1);
-		    assert_ptr(blob, "unescape_argv");
 
+		if (*filename == '&') {
+		    fp = fdopen(atoi(filename + sizeof(char)), "r");
+		} else {
 		    fp = fopen(filename, "r");
-		    if (fp != 0) {
-			length = fread(blob, sizeof(char), sb.st_size, fp);
-			fclose(fp);
-		    } else {
-			length = 0;
-		    }
+		}
+
+		if (fp) {
+		    blob = NULL;
+		    length = 0;
+		    do {
+			blob = dlg_realloc(char, length + BUFSIZ + 1, blob);
+			assert_ptr(blob, "unescape_argv");
+			bytes_read = fread(blob + length,
+					   sizeof(char),
+					   BUFSIZ,
+					   fp);
+			length += bytes_read;
+			if (ferror(fp))
+			    dlg_exiterr("error on filehandle in unescape_argv");
+		    } while (bytes_read == BUFSIZ);
+		    fclose(fp);
+
 		    blob[length] = '\0';
 
 		    list = string_to_argv(blob);
@@ -741,13 +758,19 @@ static int
 call_checklist(CALLARGS)
 {
     int tags = howmany_tags(av + 5, CHECKBOX_TAGS);
+    int code;
+    bool save_quoted = dialog_vars.quoted;
+
+    dialog_vars.quoted = !dialog_vars.separate_output;
     *offset_add = 5 + tags * CHECKBOX_TAGS;
-    return dialog_checklist(t,
+    code = dialog_checklist(t,
 			    av[1],
 			    numeric_arg(av, 2),
 			    numeric_arg(av, 3),
 			    numeric_arg(av, 4),
 			    tags, av + 5, FLAG_CHECK);
+    dialog_vars.quoted = save_quoted;
+    return code;
 }
 
 static int
@@ -1106,7 +1129,7 @@ Help(void)
     static const char *const tbl_1[] =
     {
 	"cdialog (ComeOn Dialog!) version %s",
-	"Copyright 2000-2006,2007 Thomas E. Dickey",
+	"Copyright 2000-2007,2008 Thomas E. Dickey",
 	"This is free software; see the source for copying conditions.  There is NO",
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
 	"",
@@ -1185,7 +1208,6 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	case o_backtitle:
 	    dialog_vars.backtitle = optionString(argv, &offset);
 	    break;
-	case o_separator:
 	case o_separate_widget:
 	    dialog_state.separate_str = optionString(argv, &offset);
 	    break;
@@ -1207,6 +1229,12 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	case o_nocancel:
 	    dialog_vars.nocancel = TRUE;
 	    break;
+	case o_nook:
+	    dialog_vars.nook = TRUE;
+	    break;
+	case o_quoted:
+	    dialog_vars.quoted = TRUE;
+	    break;
 	case o_single_quoted:
 	    dialog_vars.single_quoted = TRUE;
 	    break;
@@ -1275,6 +1303,13 @@ process_common_options(int argc, char **argv, int offset, bool output)
 		fprintf(stdout, "Version: %s\n", dialog_version());
 	    }
 	    break;
+	case o_separator:
+	case o_output_separator:
+	    dialog_vars.output_separator = optionString(argv, &offset);
+	    break;
+	case o_column_separator:
+	    dialog_vars.column_separator = optionString(argv, &offset);
+	    break;
 	case o_tab_correct:
 	    dialog_vars.tab_correct = TRUE;
 	    break;
@@ -1426,7 +1461,7 @@ main(int argc, char *argv[])
     int offset = 1;
     int offset_add;
     int retval = DLG_EXIT_OK;
-    int j;
+    int j, have;
     eOptions code;
     const Mode *modePtr;
     char my_buffer[MAX_LEN + 1];
@@ -1538,22 +1573,6 @@ main(int argc, char *argv[])
 
     init_dialog(dialog_state.input, dialog_state.output);
 
-    /*
-     * Trim whitespace from non-title option values, e.g., the ones that
-     * will be used as captions or prompts.
-     */
-    for (j = 1; j < argc; j++) {
-	switch (lookupOption(argv[j - 1], 7)) {
-	case o_unknown:
-	case o_title:
-	case o_backtitle:
-	    break;
-	default:
-	    dlg_trim_string(argv[j]);
-	    break;
-	}
-    }
-
     while (offset < argc && !esc_pressed) {
 	init_result(my_buffer);
 
@@ -1590,20 +1609,41 @@ main(int argc, char *argv[])
 	    Usage(temp);
 	}
 
-	if (arg_rest(&argv[offset]) < modePtr->argmin) {
+	have = arg_rest(&argv[offset]);
+	if (have < modePtr->argmin) {
 	    sprintf(temp, "Expected at least %d tokens for %.20s, have %d",
 		    modePtr->argmin - 1, argv[offset],
-		    arg_rest(&argv[offset]) - 1);
+		    have - 1);
 	    Usage(temp);
 	}
-	if (modePtr->argmax && arg_rest(&argv[offset]) > modePtr->argmax) {
+	if (modePtr->argmax && have > modePtr->argmax) {
 	    sprintf(temp,
 		    "Expected no more than %d tokens for %.20s, have %d",
 		    modePtr->argmax - 1, argv[offset],
-		    arg_rest(&argv[offset]) - 1);
+		    have - 1);
 	    Usage(temp);
 	}
 
+	/*
+	 * Trim whitespace from non-title option values, e.g., the ones that
+	 * will be used as captions or prompts.   Do that only for the widget
+	 * we are about to process, since the "--trim" option is reset before
+	 * accumulating options for each widget.
+	 */
+	for (j = offset + 1; j <= offset + have; j++) {
+	    switch (lookupOption(argv[j - 1], 7)) {
+	    case o_unknown:
+	    case o_title:
+	    case o_backtitle:
+		break;
+	    default:
+		if (argv[j] != 0) {
+		    dlg_trim_string(argv[j]);
+		}
+		break;
+	    }
+	}
+
 	retval = show_result((*(modePtr->jumper)) (dialog_vars.title,
 						   argv + offset,
 						   &offset_add));
