@@ -1,9 +1,9 @@
 /*
- *  $Id: util.c,v 1.180 2007/04/08 18:16:45 tom Exp $
+ *  $Id: util.c,v 1.196 2008/08/20 00:26:37 tom Exp $
  *
  *  util.c -- miscellaneous utilities for dialog
  *
- *  Copyright 2000-2006,2007	Thomas E. Dickey
+ *  Copyright 2000-2007,2008	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -352,6 +352,10 @@ dlg_color_setup(void)
     if (has_colors()) {		/* Terminal supports color? */
 	(void) start_color();
 
+#if defined(HAVE_USE_DEFAULT_COLORS)
+	use_default_colors();
+#endif
+
 #if defined(__NetBSD__) && defined(_CURSES_)
 #define C_ATTR(x,y) (((x) != 0 ? A_BOLD :  0) | COLOR_PAIR((y)))
 	/* work around bug in NetBSD curses */
@@ -942,6 +946,10 @@ dlg_auto_sizefile(const char *title,
     }
     if ((*height != 0) && (*width != 0)) {
 	(void) fclose(fd);
+	if (*width > SCOLS)
+	    *width = SCOLS;
+	if (*height > SLINES)
+	    *height = SLINES;
 	return;
     }
 
@@ -1020,6 +1028,25 @@ dlg_draw_box(WINDOW *win, int y, int x, int height, int width,
 }
 
 #ifdef HAVE_COLOR
+/*
+ * Draw a shadow on the parent window corresponding to the right- and
+ * bottom-edge of the child window, to give a 3-dimensional look.
+ */
+static void
+draw_childs_shadow(WINDOW *parent, WINDOW *child)
+{
+    if (has_colors()) {		/* Whether terminal supports color? */
+	chtype save = getattrs(parent);
+
+	dlg_draw_shadow(parent,
+			getbegy(child) - getbegy(parent),
+			getbegx(child) - getbegx(parent),
+			getmaxy(child),
+			getmaxx(child));
+	wattrset(parent, save);
+    }
+}
+
 /*
  * Draw shadows along the right and bottom edge to give a more 3D look
  * to the boxes
@@ -1034,27 +1061,20 @@ dlg_draw_shadow(WINDOW *win, int y, int x, int height, int width)
 	for (i = 0; i < SHADOW_ROWS; ++i) {
 	    for (j = 0; j < width; ++j) {
 		if (wmove(win, i + y + height, j + x + SHADOW_COLS) != ERR) {
-		    (void) waddch(win, CharOf(winch(win)));
+		    (void) waddch(win, winch(win) & (chtype) (~A_COLOR));
 		}
 	    }
 	}
 	for (i = 0; i < height; i++) {
 	    for (j = 0; j < SHADOW_COLS; ++j) {
 		if (wmove(win, i + y + SHADOW_ROWS, j + x + width) != ERR) {
-		    (void) waddch(win, CharOf(winch(win)));
+		    (void) waddch(win, winch(win) & (chtype) (~A_COLOR));
 		}
 	    }
 	}
 	(void) wnoutrefresh(win);
     }
 }
-
-#define dlg_draw_shadow_box(win, height, width) \
-	dlg_draw_shadow((win), \
-			(height) - SHADOW_ROWS, \
-			(width) - SHADOW_COLS, \
-			(height), \
-			(width))
 #endif /* HAVE_COLOR */
 
 /*
@@ -1126,9 +1146,11 @@ dlg_exit(int code)
 	    fclose(dialog_state.input);
 	    dialog_state.input = 0;
 	}
-	if (dialog_state.pipe_input != stdin) {
-	    fclose(dialog_state.pipe_input);
-	    dialog_state.pipe_input = 0;
+	if (dialog_state.pipe_input) {
+	    if (dialog_state.pipe_input != stdin) {
+		fclose(dialog_state.pipe_input);
+		dialog_state.pipe_input = 0;
+	    }
 	}
 	_exit(code);
     }
@@ -1416,8 +1438,6 @@ dlg_del_window(WINDOW *win)
     }
 
     if (q) {
-	if (q->shadow != 0)
-	    delwin(q->shadow);
 	delwin(q->normal);
 	dlg_unregister_window(q->normal);
 	free(q);
@@ -1434,16 +1454,6 @@ dlg_new_window(int height, int width, int y, int x)
     WINDOW *win;
     DIALOG_WINDOWS *p = dlg_calloc(DIALOG_WINDOWS, 1);
 
-#ifdef HAVE_COLOR
-    if (dialog_state.use_shadow) {
-	if ((win = newwin(height, width,
-			  y + SHADOW_ROWS,
-			  x + SHADOW_COLS)) != 0) {
-	    dlg_draw_shadow_box(win, height, width);
-	}
-	p->shadow = win;
-    }
-#endif
     if ((win = newwin(height, width, y, x)) == 0) {
 	dlg_exiterr("Can't make new window at (%d,%d), size (%d,%d).\n",
 		    y, x, height, width);
@@ -1451,6 +1461,35 @@ dlg_new_window(int height, int width, int y, int x)
     p->next = dialog_state.all_windows;
     p->normal = win;
     dialog_state.all_windows = p;
+#ifdef HAVE_COLOR
+    if (dialog_state.use_shadow) {
+	draw_childs_shadow(p->shadow = stdscr, win);
+    }
+#endif
+
+    (void) keypad(win, TRUE);
+    return win;
+}
+
+WINDOW *
+dlg_new_modal_window(WINDOW *parent, int height, int width, int y, int x)
+{
+    WINDOW *win;
+    DIALOG_WINDOWS *p = dlg_calloc(DIALOG_WINDOWS, 1);
+
+    (void) parent;
+    if ((win = newwin(height, width, y, x)) == 0) {
+	dlg_exiterr("Can't make new window at (%d,%d), size (%d,%d).\n",
+		    y, x, height, width);
+    }
+    p->next = dialog_state.all_windows;
+    p->normal = win;
+    dialog_state.all_windows = p;
+#ifdef HAVE_COLOR
+    if (dialog_state.use_shadow) {
+	draw_childs_shadow(p->shadow = parent, win);
+    }
+#endif
 
     (void) keypad(win, TRUE);
     return win;
@@ -1481,10 +1520,8 @@ dlg_move_window(WINDOW *win, int height, int width, int y, int x)
 #ifdef HAVE_COLOR
 	    if (p->shadow != 0) {
 		if (dialog_state.use_shadow) {
-		    (void) wresize(p->shadow, height, width);
 		    (void) mvwin(p->shadow, y + SHADOW_ROWS, x + SHADOW_COLS);
 		} else {
-		    delwin(p->shadow);
 		    p->shadow = 0;
 		}
 	    }
@@ -1493,7 +1530,7 @@ dlg_move_window(WINDOW *win, int height, int width, int y, int x)
 
 #ifdef HAVE_COLOR
 	    if (p->shadow)
-		dlg_draw_shadow_box(p->shadow, height, width);
+		draw_childs_shadow(p->shadow, win);
 #endif
 	}
     }
@@ -1803,8 +1840,44 @@ dlg_add_result(const char *string)
     strcat(dialog_vars.input_result, string);
 }
 
-#define FIX_SINGLE "\n\\'"
-#define FIX_DOUBLE "\n\\\"[]{}?*;`~#$^&()|<>\t "
+/*
+ * These are characters that (aside from the quote-delimiter) will have to
+ * be escaped in a single- or double-quoted string.
+ */
+#define FIX_SINGLE "\n\\"
+#define FIX_DOUBLE FIX_SINGLE "[]{}?*;`~#$^&()|<>"
+
+/*
+ * Returns the quote-delimiter.
+ */
+static const char *
+quote_delimiter(void)
+{
+    return dialog_vars.single_quoted ? "'" : "\"";
+}
+
+/*
+ * Returns true if we should quote the given string.
+ */
+static bool
+must_quote(char *string)
+{
+    bool code = FALSE;
+
+    if (*string != '\0') {
+	unsigned len = strlen(string);
+	if (strcspn(string, quote_delimiter()) != len)
+	    code = TRUE;
+	else if (strcspn(string, "\n\t ") != len)
+	    code = TRUE;
+	else
+	    code = (strcspn(string, FIX_DOUBLE) != len);
+    } else {
+	code = TRUE;
+    }
+
+    return code;
+}
 
 /*
  * Add a quoted string to the result buffer.
@@ -1813,28 +1886,79 @@ void
 dlg_add_quoted(char *string)
 {
     char temp[2];
-    const char *my_quote = dialog_vars.single_quoted ? "'" : "\"";
+    const char *my_quote = quote_delimiter();
     const char *must_fix = (dialog_vars.single_quoted
 			    ? FIX_SINGLE
 			    : FIX_DOUBLE);
 
-    if (dialog_vars.single_quoted
-	&& strlen(string) != 0
-	&& strcspn(string, FIX_DOUBLE FIX_SINGLE) == strlen(string)) {
-	dlg_add_result(string);
-    } else {
+    if (dialog_vars.quoted || must_quote(string)) {
 	temp[1] = '\0';
 	dlg_add_result(my_quote);
 	while (*string != '\0') {
 	    temp[0] = *string++;
-	    if (strchr(must_fix, *temp) != 0)
+	    if (strchr(my_quote, *temp) || strchr(must_fix, *temp))
 		dlg_add_result("\\");
 	    dlg_add_result(temp);
 	}
 	dlg_add_result(my_quote);
+    } else {
+	dlg_add_result(string);
     }
 }
 
+/*
+ * When adding a result, make that depend on whether "--quoted" is used.
+ */
+void
+dlg_add_string(char *string)
+{
+    if (dialog_vars.quoted) {
+	dlg_add_quoted(string);
+    } else {
+	dlg_add_result(string);
+    }
+}
+
+bool
+dlg_need_separator(void)
+{
+    bool result = FALSE;
+
+    if (dialog_vars.output_separator) {
+	result = TRUE;
+    } else if (dialog_vars.input_result && *(dialog_vars.input_result)) {
+	result = TRUE;
+    }
+    return result;
+}
+
+void
+dlg_add_separator(void)
+{
+    const char *separator = (dialog_vars.separate_output) ? "\n" : " ";
+
+    if (dialog_vars.output_separator)
+	separator = dialog_vars.output_separator;
+
+    dlg_add_result(separator);
+}
+
+/*
+ * Some widgets support only one value of a given variable - save/restore the
+ * global dialog_vars so we can override it consistently.
+ */
+void
+dlg_save_vars(DIALOG_VARS * vars)
+{
+    *vars = dialog_vars;
+}
+
+void
+dlg_restore_vars(DIALOG_VARS * vars)
+{
+    dialog_vars = *vars;
+}
+
 /*
  * Called each time a widget is invoked which may do output, increment a count.
  */
